/*=============================================================================
    Copyright (c) 2019 Nikita Kniazev

    Distributed under the Boost Software License, Version 1.0. (See accompanying
    file LICENSE_1_0.txt or copy at http://www.boost.org/LICENSE_1_0.txt)
=============================================================================*/
#ifdef BOOST_BUILD_PCH_ENABLED

#include <boost/spirit/include/karma.hpp>
#include <boost/spirit/include/support_argument.hpp>
#include <boost/spirit/include/phoenix_core.hpp>
#include <boost/spirit/include/phoenix_operator.hpp>
#include <boost/core/lightweight_test.hpp>
#include <boost/optional.hpp>
#include <boost/variant.hpp>
#include <vector>
#include <string>
#include <sstream>
#include <iostream>

#endif
